import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import ddf.minim.spi.*; 
import ddf.minim.signals.*; 
import ddf.minim.*; 
import ddf.minim.analysis.*; 
import ddf.minim.ugens.*; 
import ddf.minim.effects.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class ConnectedWorlds extends PApplet {








int WIDTH = 896;
int HEIGHT = 640;
int tile_size = 64;

Player player;
Tilemap tilemap;
Minim minim;
AudioPlayer song;
AudioSample jump;
AudioSample pickup;
AudioSample enter;

PImage darkmatter;
int score;
int deaths;
boolean gameOver = false;
public boolean[] keys;

public void setup() {
  size(WIDTH, HEIGHT);
  minim = new Minim(this);
  song = minim.loadFile("Desert.mp3");
  jump = minim.loadSample("Jump.wav");
  jump.setGain(-11);
  pickup = minim.loadSample("Pickup.wav");
  enter = minim.loadSample("EnterPortal.wav");
  song.play();
  song.loop();
  player = new Player();
  tilemap = new Tilemap(WIDTH, HEIGHT, tile_size);
  keys = new boolean[4];
  darkmatter = loadImage("DarkMatter.png");
  score = 0;
  deaths = 0;
}


public void draw() {
  if(gameOver == false) {
    update();
    render();
  } else {
    background(0);
    textSize(16);
    fill(255);
    text("Thank you for playing!", 364,320);
  }
}

public void update() {
  player.update();
  println("X: " + PApplet.parseInt(player.x+WIDTH/2-player.player_size/2) + " Y: " + PApplet.parseInt(player.y+HEIGHT/2-player.player_size/2));
}

public void render() {
  tilemap.render();
}

public void keyPressed() {
  if(keyCode == UP) keys[3] = true;
  if(keyCode == LEFT) keys[1] = true;
  if(keyCode == RIGHT) keys[2] = true;
  if(keyCode == DOWN) keys[0] = true;
}

public void keyReleased() {
  if(keyCode == UP) keys[3] = false;
  if(keyCode == LEFT) keys[1] = false;
  if(keyCode == RIGHT) keys[2] = false;
  if(keyCode == DOWN) keys[0] = false;
}
public class Map {
  int map_w;
  int map_h;
  int level;
  int map[][];
  int spawnX, spawnY;
  int totalPoints;
  Portal portal;
  ArrayList<Pickup> pickups;
  int[] colors = {
    0xff4CFF00, 0xff7F3300, 0xff267F00, 0xff542709, 0xff303030, 0xffA0A0A0
  };
  PImage levelImage;
  public Map(int level) {
    this.level = level;
    levelImage = loadImage("levels/"+level+".png");
    this.map_w = levelImage.width;
    this.map_h = levelImage.height;
    this.map = new int[map_w][map_h];
    pickups = new ArrayList<Pickup>();
    for (int x = 0; x < map_w; x++) {
      for (int y = 0; y < map_h; y++) {
        int pixel = levelImage.get(x, y);
        boolean choose = false;
        for (int i = 0; i < colors.length; i++) {
          //println("C1: " + pixel + " C2: " + colors[i]);
          if (choose == false) {
            if (pixel == colors[i]) {
              choose = true;
              map[x][y] = i;
            } else {
              map[x][y] = -1;
            }
          }
        }
        if(pixel == color(0)) pickups.add(new Pickup(x*tile_size, y*tile_size));
        if(pixel == color(255,0,0)) {
          spawnX = x;
          spawnY = y;
        }
        if(pixel == color(0,0,255)) {
          portal = new Portal(x*tile_size-tile_size, y*tile_size-tile_size);
        }
      }
    }
    totalPoints = pickups.size();
  }
}

public class Pickup {
  int x;
  int y;
  public Pickup(int x, int y) {
    this.x = x-tile_size;
    this.y = y-tile_size;
  }
  
  public boolean update() {
    image(darkmatter, x-tilemap.camx+8, y-tilemap.camy+8);
    if(player.x+WIDTH/2 > x && player.x+WIDTH/2 < x+48 && player.y+HEIGHT/2 > y && player.y+HEIGHT/2 < y+48) {
      return true;
    } else {
      return false;
    }
  }
  
}
public class Player {
  float x;
  float y;
  float xv;
  float yv;
  int action;
  int frame;
  int player_size;
  PImage frames[];
  PImage frames2[];
  int timer;
  boolean right;
  public Player() {
    this.x = 0;
    this.y = 0;
    this.xv = 0;
    this.yv = 0;
    this.player_size = 60;
    this.right = true;
    PImage image = loadImage("Chicken.png");
    PImage image2 = loadImage("Chicken2.png");
    frames = new PImage[image.width/player_size];
    frames2 = new PImage[image.width/player_size];
    for (int n = 0; n < frames.length; n++) {
      frames[n] = image.get(n*64, 0, 64, 64);
    }
    for (int n = 0; n < frames2.length; n++) {
      frames2[n] = image2.get(n*64, 0, 64, 64);
    }
  }

  public void update() {
    float oldx = x;
    float oldy = y;
    if (keys[1] == true) xv-=0.52f;
    if (keys[2] == true) xv+=0.52f;
    if (keys[3] == true && yv == 0) {
      yv-=12;
      jump.trigger();
    }
    if (xv > 0) right = true;
    if (xv < 0) right = false;
    xv/=1.1f;
    yv+=0.4f;
    y+=yv;
    boolean collided = false;
    for (int stepx = 0; stepx < 2; stepx++) {
      for (int stepy = 0; stepy < 2; stepy++) {
        int cornerx = floor((x+WIDTH/2-player_size/2+player_size*stepx)/tile_size)+1;
        int cornery = floor((y+HEIGHT/2-player_size/2+player_size*stepy)/tile_size)+1;
        if (cornerx < tilemap.map.map_w && cornerx >= 0 && cornery < tilemap.map.map_h && cornery >= 0) {
          if (tilemap.map.map[cornerx][cornery] != -1 && tilemap.map.map[cornerx][cornery] != 2 && tilemap.map.map[cornerx][cornery] != 3) {
            collided = true;
          }
        }
      }
    }
    if (collided == true) {
      y = oldy;
      if(yv < 0) {
        yv = 0.01f;
      } else {
        yv = 0;
      }
    }
    x+=xv;
    collided = false;
    for (int stepx = 0; stepx < 2; stepx++) {
      for (int stepy = 0; stepy < 2; stepy++) {
        int cornerx = floor((x+WIDTH/2-player_size/2+player_size*stepx)/tile_size)+1;
        int cornery = floor((y+HEIGHT/2-player_size/2+player_size*stepy)/tile_size)+1;
        if (cornerx < tilemap.map.map_w && cornerx >= 0 && cornery < tilemap.map.map_h && cornery >= 0) {
          if (tilemap.map.map[cornerx][cornery] != -1 && tilemap.map.map[cornerx][cornery] != 2 && tilemap.map.map[cornerx][cornery] != 3) {
            collided = true;
          }
        }
      }
    }
    if (collided == true) {
      x = oldx;
      xv = 0;
    }
    if (xv < 0.1f && xv > -0.1f) xv = 0;
    if (xv == 0) {
      frame = 0;
    }
    if(xv != 0) {
      if (millis() - timer >= 125) {
        if (frame == 2) {
          frame = 0;
        } 
        else if (frame == 1) {
          frame = 2;
        } 
        else if (frame == 0) {
          frame = 1;
        } else if (frame == 3) {
          frame = 0;
        }
        timer = millis();
      }
    }
    if (oldy != y) {
      frame = 3;
    }
    if(y > tilemap.map.map_h*tile_size) {
      death();
    }
  }
  
  public void death() {
    deaths++;
    tilemap.loadMap(tilemap.current_level);
  }
}

public class Portal {
  int x, y;
  boolean open;
  PImage portalopen, portalclosed;
  public Portal(int x, int y) {
    this.x = x;
    this.y = y;
    portalopen = loadImage("PortalOpen.png");
    portalclosed = loadImage("PortalClosed.png");
    open = false;
  }

  public boolean update() {
    if (score >= tilemap.map.totalPoints) {
      open = true;
    }
    if (open) {
      image(portalopen, x-tilemap.camx, y-tilemap.camy);
      if (player.x+WIDTH/2 > x && player.x+WIDTH/2 < x+64 && player.y+HEIGHT/2 > y && player.y+HEIGHT/2 < y+64) {
        return true;
      } else {
        return false;
      }
    } else {
      image(portalclosed, x-tilemap.camx, y-tilemap.camy);
      return false;
    }
  }
}

public class Tilemap {
  int WIDTH;
  int HEIGHT;
  int tile_size;
  int map_display_w;
  int map_display_h;
  int map_display_buffer;
  int current_level;
  float camx;
  float camy;
  PImage[] tiles;
  PImage background;
  Map[] levels;
  Map map;
  public Tilemap(int WIDTH, int HEIGHT, int TILE_SIZE) {
    this.WIDTH = WIDTH;
    this.HEIGHT = HEIGHT;
    this.tile_size = TILE_SIZE;
    this.map_display_w = PApplet.parseInt(WIDTH/tile_size);
    this.map_display_h = PApplet.parseInt(HEIGHT/tile_size);
    this.map_display_buffer = 2;
    this.current_level = 0;
    background = loadImage("background.png");
    PImage tileset = loadImage("Tiles.png");
    tiles = new PImage[tileset.width/tile_size];
    for (int tile = 0; tile < tiles.length; tile++) {
      tiles[tile] = tileset.get(tile*tile_size, 0, tile_size, tile_size);
    }
    levels = new Map[4];
    for (int i = 0; i < levels.length; i++) {
      levels[i] = new Map(i);
    }
    loadMap(current_level);
  }


  public void render() {
    image(background, 0, 0);
    camx = constrain(player.x, 0, (map.map_w-1)*tile_size-WIDTH);
    camy = constrain(player.y, 0, (map.map_h-1)*tile_size-HEIGHT);
    int offset_x = PApplet.parseInt(camx) % tile_size;
    int offset_y = PApplet.parseInt(camy) % tile_size;
    int first_tile_x = floor(camx/tile_size);
    int first_tile_y = floor(camy/tile_size);
    for (int y = 0; y < map_display_h + map_display_buffer; y++) {
      for (int x = 0; x < map_display_w + map_display_buffer; x++) {
        if (x+first_tile_x >= 0 && x+first_tile_x < map.map_w && y+first_tile_y >= 0 && y+first_tile_y < map.map_h) {
          if (map.map[x+first_tile_x][y+first_tile_y] != -1) {
            //println("X:" + int(x+first_tile_x) + " Y:" + int(y+first_tile_y) + " Map:" + int(map.map[x+first_tile_x][y+first_tile_y]));
            image(tiles[map.map[x+first_tile_x][y+first_tile_y]], ((x-1)*tile_size)-offset_x, ((y-1)*tile_size)-offset_y);
          }
        }
      }
    }
    for (int i = map.pickups.size()-1; i >= 0; i--) {
      Pickup p = map.pickups.get(i);
      if (p.update() == true) {
        map.pickups.remove(i);
        pickup.trigger();
        score++;
      }
    }
    if (player.right == true) {
      image(player.frames[player.frame], player.x-camx+WIDTH/2-tile_size/2, player.y-camy+HEIGHT/2-tile_size/2);
    } 
    else {
      image(player.frames2[player.frame], player.x-camx+WIDTH/2-tile_size/2, player.y-camy+HEIGHT/2-tile_size/2);
    }
    if (map.portal.update() == true) {
      enter.trigger();
      score = 0;
      current_level++;
      loadMap(current_level);
    }
    textSize(20);
    fill(0);
    text("Dark Matter: " + score + "/" + map.totalPoints, 5, 20);
    text("Deaths: " + deaths, WIDTH-110, 20);
    textSize(40);
    if (millis() < 5000) {
      text("Find all the dark matter to open the portal", 28, HEIGHT-40);
    }
  }

  public void loadMap(int level) {
    if (level >= 4) {
      gameOver = true;
    } 
    else {
      map = levels[level];
      player.x = map.spawnX*tile_size-WIDTH/2;
      player.y = map.spawnY*tile_size-HEIGHT/2;
      player.xv = 0;
      player.yv = 0;
    }
  }
}

  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "ConnectedWorlds" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
