public class Tilemap {
  int WIDTH;
  int HEIGHT;
  int tile_size;
  int map_display_w;
  int map_display_h;
  int map_display_buffer;
  int current_level;
  float camx;
  float camy;
  PImage[] tiles;
  PImage background;
  Map[] levels;
  Map map;
  public Tilemap(int WIDTH, int HEIGHT, int TILE_SIZE) {
    this.WIDTH = WIDTH;
    this.HEIGHT = HEIGHT;
    this.tile_size = TILE_SIZE;
    this.map_display_w = int(WIDTH/tile_size);
    this.map_display_h = int(HEIGHT/tile_size);
    this.map_display_buffer = 2;
    this.current_level = 0;
    background = loadImage("background.png");
    PImage tileset = loadImage("Tiles.png");
    tiles = new PImage[tileset.width/tile_size];
    for (int tile = 0; tile < tiles.length; tile++) {
      tiles[tile] = tileset.get(tile*tile_size, 0, tile_size, tile_size);
    }
    levels = new Map[4];
    for (int i = 0; i < levels.length; i++) {
      levels[i] = new Map(i);
    }
    loadMap(current_level);
  }


  public void render() {
    image(background, 0, 0);
    camx = constrain(player.x, 0, (map.map_w-1)*tile_size-WIDTH);
    camy = constrain(player.y, 0, (map.map_h-1)*tile_size-HEIGHT);
    int offset_x = int(camx) % tile_size;
    int offset_y = int(camy) % tile_size;
    int first_tile_x = floor(camx/tile_size);
    int first_tile_y = floor(camy/tile_size);
    for (int y = 0; y < map_display_h + map_display_buffer; y++) {
      for (int x = 0; x < map_display_w + map_display_buffer; x++) {
        if (x+first_tile_x >= 0 && x+first_tile_x < map.map_w && y+first_tile_y >= 0 && y+first_tile_y < map.map_h) {
          if (map.map[x+first_tile_x][y+first_tile_y] != -1) {
            //println("X:" + int(x+first_tile_x) + " Y:" + int(y+first_tile_y) + " Map:" + int(map.map[x+first_tile_x][y+first_tile_y]));
            image(tiles[map.map[x+first_tile_x][y+first_tile_y]], ((x-1)*tile_size)-offset_x, ((y-1)*tile_size)-offset_y);
          }
        }
      }
    }
    for (int i = map.pickups.size()-1; i >= 0; i--) {
      Pickup p = map.pickups.get(i);
      if (p.update() == true) {
        map.pickups.remove(i);
        pickup.trigger();
        score++;
      }
    }
    if (player.right == true) {
      image(player.frames[player.frame], player.x-camx+WIDTH/2-tile_size/2, player.y-camy+HEIGHT/2-tile_size/2);
    } 
    else {
      image(player.frames2[player.frame], player.x-camx+WIDTH/2-tile_size/2, player.y-camy+HEIGHT/2-tile_size/2);
    }
    if (map.portal.update() == true) {
      enter.trigger();
      score = 0;
      current_level++;
      loadMap(current_level);
    }
    textSize(20);
    fill(0);
    text("Dark Matter: " + score + "/" + map.totalPoints, 5, 20);
    text("Deaths: " + deaths, WIDTH-110, 20);
    textSize(40);
    if (millis() < 5000) {
      text("Find all the dark matter to open the portal", 28, HEIGHT-40);
    }
  }

  public void loadMap(int level) {
    if (level >= 4) {
      gameOver = true;
    } 
    else {
      map = levels[level];
      player.x = map.spawnX*tile_size-WIDTH/2;
      player.y = map.spawnY*tile_size-HEIGHT/2;
      player.xv = 0;
      player.yv = 0;
    }
  }
}

